@@ -3,6 +3,9 @@
 #include "XSUB.h"
 #include "slot_compat.h"
 
+/* PADNAMEf_CONST - compile-time constant flag (0x40 is unused in standard perl) */
+#define PADNAMEf_CONST 0x40
+
 /* Custom op support */
 static XOP slot_get_xop;
 static XOP slot_set_xop;
@@ -199,9 +202,11 @@ static OP* slot_call_checker(pTHX_ OP *entersubop, GV *namegv, SV *ckobj) {
     return entersubop;
 }
 
-/* Call checker for slot::get('constant_name') - optimize to custom op */
+/* Call checker for slot::get('constant_name') - optimize to custom op
+ * Detects both literal strings and const::c() variables (via PADNAMEf_CONST) */
 static OP* slot_get_call_checker(pTHX_ OP *entersubop, GV *namegv, SV *ckobj) {
     OP *pushop, *cvop, *argop;
+    SV *namesv = NULL;
     PERL_UNUSED_ARG(namegv);
     PERL_UNUSED_ARG(ckobj);
 
@@ -215,29 +220,53 @@ static OP* slot_get_call_checker(pTHX_ OP *entersubop, GV *namegv, SV *ckobj) {
         cvop = OpSIBLING(cvop);
     }
 
-    /* Check if single constant string argument */
-    if (argop != cvop && OpSIBLING(argop) == cvop && argop->op_type == OP_CONST) {
-        SV *namesv = cSVOPx(argop)->op_sv;
-        if (SvPOK(namesv)) {
-            STRLEN name_len;
-            const char *name = SvPV(namesv, name_len);
-            SV **svp = hv_fetch(g_slot_index, name, name_len, 0);
-            if (svp) {
-                IV idx = SvIV(*svp);
-                OP *newop = newOP(OP_CUSTOM, 0);
-                newop->op_ppaddr = pp_slot_get;
-                newop->op_targ = idx;
-                op_free(entersubop);
-                return newop;
+    /* Must be single argument */
+    if (argop == cvop || OpSIBLING(argop) != cvop)
+        return entersubop;
+
+    /* Check for literal string constant (OP_CONST) */
+    if (argop->op_type == OP_CONST) {
+        namesv = cSVOPx(argop)->op_sv;
+    }
+    /* Check for const::c() variable (OP_PADSV with PADNAMEf_CONST flag) */
+    else if (argop->op_type == OP_PADSV) {
+        PADOFFSET po = argop->op_targ;
+        if (PL_comppad_name && po < PadnamelistMAX(PL_comppad_name) + 1) {
+            PADNAME *pn = PadnamelistARRAY(PL_comppad_name)[po];
+            if (pn && (PadnameFLAGS(pn) & PADNAMEf_CONST)) {
+                /* Has our const flag - get value from compilation pad */
+                if (PL_comppad) {
+                    SV **svp = av_fetch(PL_comppad, po, 0);
+                    if (svp && SvPOK(*svp)) {
+                        namesv = *svp;
+                    }
+                }
             }
         }
     }
+
+    /* If we got a slot name, optimize to custom op */
+    if (namesv && SvPOK(namesv)) {
+        STRLEN name_len;
+        const char *name = SvPV(namesv, name_len);
+        SV **svp = hv_fetch(g_slot_index, name, name_len, 0);
+        if (svp) {
+            IV idx = SvIV(*svp);
+            OP *newop = newOP(OP_CUSTOM, 0);
+            newop->op_ppaddr = pp_slot_get;
+            newop->op_targ = idx;
+            op_free(entersubop);
+            return newop;
+        }
+    }
     return entersubop;
 }
 
-/* Call checker for slot::set('constant_name', $val) - optimize to custom op */
+/* Call checker for slot::set('constant_name', $val) - optimize to custom op
+ * Detects both literal strings and const::c() variables (via PADNAMEf_CONST) */
 static OP* slot_set_call_checker(pTHX_ OP *entersubop, GV *namegv, SV *ckobj) {
     OP *pushop, *cvop, *argop, *valop;
+    SV *namesv = NULL;
     PERL_UNUSED_ARG(namegv);
     PERL_UNUSED_ARG(ckobj);
 
@@ -251,31 +280,51 @@ static OP* slot_set_call_checker(pTHX_ OP *entersubop, GV *namegv, SV *ckobj) {
         cvop = OpSIBLING(cvop);
     }
 
-    /* Check if two args, first is constant string */
+    /* Must be two args */
     valop = OpSIBLING(argop);
-    if (argop != cvop && valop && valop != cvop && OpSIBLING(valop) == cvop 
-        && argop->op_type == OP_CONST) {
-        SV *namesv = cSVOPx(argop)->op_sv;
-        if (SvPOK(namesv)) {
-            STRLEN name_len;
-            const char *name = SvPV(namesv, name_len);
-            SV **svp = hv_fetch(g_slot_index, name, name_len, 0);
-            if (svp) {
-                IV idx = SvIV(*svp);
-                OP *newop;
-                
-                /* Detach value arg from list */
-                OpMORESIB_set(pushop, cvop);
-                OpLASTSIB_set(valop, NULL);
-                
-                newop = newUNOP(OP_CUSTOM, 0, valop);
-                newop->op_ppaddr = pp_slot_set;
-                newop->op_targ = idx;
-                op_free(entersubop);
-                return newop;
+    if (argop == cvop || !valop || valop == cvop || OpSIBLING(valop) != cvop)
+        return entersubop;
+
+    /* Check for literal string constant (OP_CONST) */
+    if (argop->op_type == OP_CONST) {
+        namesv = cSVOPx(argop)->op_sv;
+    }
+    /* Check for const::c() variable (OP_PADSV with PADNAMEf_CONST flag) */
+    else if (argop->op_type == OP_PADSV) {
+        PADOFFSET po = argop->op_targ;
+        if (PL_comppad_name && po < PadnamelistMAX(PL_comppad_name) + 1) {
+            PADNAME *pn = PadnamelistARRAY(PL_comppad_name)[po];
+            if (pn && (PadnameFLAGS(pn) & PADNAMEf_CONST)) {
+                if (PL_comppad) {
+                    SV **svp = av_fetch(PL_comppad, po, 0);
+                    if (svp && SvPOK(*svp)) {
+                        namesv = *svp;
+                    }
+                }
             }
         }
     }
+
+    /* If we got a slot name, optimize to custom op */
+    if (namesv && SvPOK(namesv)) {
+        STRLEN name_len;
+        const char *name = SvPV(namesv, name_len);
+        SV **svp = hv_fetch(g_slot_index, name, name_len, 0);
+        if (svp) {
+            IV idx = SvIV(*svp);
+            OP *newop;
+
+            /* Detach value arg from list */
+            OpMORESIB_set(pushop, cvop);
+            OpLASTSIB_set(valop, NULL);
+
+            newop = newUNOP(OP_CUSTOM, 0, valop);
+            newop->op_ppaddr = pp_slot_set;
+            newop->op_targ = idx;
+            op_free(entersubop);
+            return newop;
+        }
+    }
     return entersubop;
 }
 
